package RPM::VersionCompare;
use strict;
use warnings;
use version 0.77; our $VERSION = qv("v0.1.1");

require Exporter;
require AutoLoader;
require XSLoader;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw(labelCompare labelSplit rpmvercmp);
sub AUTOLOAD { }

XSLoader::load(__PACKAGE__, $VERSION);

sub labelSplit {
    my $label = $_[0];
    if (!defined $label) { return undef; }

    # See http://www.rpm.org/wiki/PackagerDocs/Dependencies#RequiringPackages
    $label =~ /\A(?|([^:]+):|())([^-]*)(?|-([^-]+)|())\z/ or return undef;

    return (($1 eq '')?0:$1, $2, ($3 eq '')?0:$3);
}

sub labelCompare {
    my ($label1, $label2) = @_;

    my @compounds1 = labelSplit($label1);
    my @compounds2 = labelSplit($label2);
    if (!defined $compounds1[0] || !defined $compounds2[0]) { return undef; }

    my $resolution;
    for my $i (0..2) {
        $resolution = rpmvercmp($compounds1[$i], $compounds2[$i]);
        if ($resolution != 0) { last; }
    }
    return $resolution;
}

1;
__END__
=encoding utf8

=head1 NAME

RPM::VersionCompare - Compare RPM version strings

=head1 SYNOPSIS

  use RPM::VersionCompare;
  RPM::VersionCompare::rpmvercmp('1.0', '0.9');
  RPM::VersionCompare::labelSplit('4:5.12.0-140');
  RPM::VersionCompare::labelCompare('4:5.12.0-140', '5.14.1');

=head1 DESCRIPTION

This module provides functions to compare RPM version strings. No function is
exported by default. If possible, calls are passed to native librpm library.
Otherwise Python extension provided with RPM sources is re-implemented.

=over 8

=item B<labelCompare(LABEL1, LABEL2)>

Compares two version strings including epoch and release. Return 1 if
B<LABEL1> is bigger, return 0 if B<LABEL1> equals to B<LABEL2>, return -1
otherwise.

=item B<labelSplit(LABEL)>

Splits a version B<LABEL> (e.g. "4:5.12.0-140") into epoch, version and
release. The epoch and release parts are optional. If they miss, the separator
(collon for epoch, hyphen for release) should miss too and their implicit
value is 0. See
L<http://www.rpm.org/wiki/PackagerDocs/Dependencies#RequiringPackages> for
specification.

If LABEL is in correct format, array (EPOCH, VERSION, RELEASE) will be
returned. Otherwise undef is returned.

=item B<rpmvercmp(VERSION1, VERSION2)>

Compares two RPM version strings by calling librpm function rpmvercmp(). It
does not support strings with RPM epoch or release. (Use labelCompare() if you
need to support epoch or release.) It will return 1 if B<VERSION1> string is
bigger, 0 if strings are equal, and -1 if B<VERSION1> is lesser.

=back

=head1 AUTHOR

Petr Písař <ppisar@redhat.com>

=head1 COPYING

Copyright (C) 2011  Petr Písař <ppisar@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut
